<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Spatie\Permission\Models\Permission;
use Spatie\Permission\Models\Role;

class PermissionController extends Controller
{

    public function __construct()
    {
        $this->middleware('adminauth');
        $this->middleware('permission:permission_view', ['only' => ['index']]);
        $this->middleware('permission:permission_create', ['only' => ['create','store','assign','assignStore']]);
        $this->middleware('permission:permission_edit', ['only' => ['edit','update']]);
        $this->middleware('permission:permission_delete', ['only' => ['destroy']]);
    }

   /**
     * Display the List view.
     *
     */
    public function index()
    {
        $permission = Permission::all();
        return view('admin.permission.index', ['permission' => $permission]);
    }


    /**
     * get listing with ajax
     * @return Renderable
     */
    public function ajaxList(Request $request)
    {
        $draw = $request->get('draw');
        $start = $request->get("start");
        $rowperpage = $request->get("length"); // total number of rows per page

        $columnIndex_arr = $request->get('order');
        $columnName_arr = $request->get('columns');
        $order_arr = $request->get('order');
        $search_arr = $request->get('search');

        $columnIndex = $columnIndex_arr[0]['column']; // Column index
        $columnName = $columnName_arr[$columnIndex]['data']; // Column name
        $columnSortOrder = $order_arr[0]['dir']; // asc or desc
        $searchValue = $search_arr['value']; // Search value

        $totalRecords = Permission::select('count(*) as allcount')->count();
        $totalRecordswithFilter = Permission::select('count(*) as allcount')->where('name', 'like', '%' . $searchValue . '%')->count();

        // Get records, also we have included search filter as well
        $records = Permission::orderBy('id', 'desc')
             ->orderBy($columnName, $columnSortOrder)
            ->where('permissions.name', 'like', '%' . $searchValue . '%')
            ->select('permissions.*')
            ->skip($start)
            ->take($rowperpage)
            ->get();

        $data_arr = array();

        foreach ($records as $record) {
            $actionHtml = '<li><a class="dropdown-item" href="'.route('permission.edit', $record->id).'">Edit</a></li>';

            $data_arr[] = array(
                "id" => $record->id,
                "name" => $record->name,
                'action' => ''
            );
        }

        $response = array(
            "draw" => intval($draw),
            "iTotalRecords" => $totalRecords,
            "iTotalDisplayRecords" => $totalRecordswithFilter,
            "aaData" => $data_arr,
        );

        echo json_encode($response);
    }

    /**
     * Display the Permission Creation view.
     *
     * @return \Illuminate\View\View
     */
    public function create()
    {
        return view('admin.permission.create');
    }

    /**
     * Handle an incoming Permission request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\RedirectResponse
     *
     * @throws \Illuminate\Validation\ValidationException
     */
    public function store(Request $request)
    {
        $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'permission' => ['required'],
        ]);

        $name = strtolower($request->name);

        foreach($request->permission as $permission){
            $permission = Permission::create([
                'name' => $name.'_'.$permission,
                'guard_name' => 'web',
            ]);
        }
        if($permission){
            return redirect('admin/permission/index')->with('message', 'Permission created successfully!');
        } else {
            return back()->withError('Something went wrong! Please try again.');
        }
    }

    /**
     * Edit the Permission.
     *
     */
    public function edit(Permission $permission)
    {
        return view('admin.permission.edit', ['permission' => $permission]);
    }

    /**
     * update the Permission.
     *
     */
    public function update(Request $request, $id)
    {
        $permission = Permission::find($id);
        $permission->update($request->all());
        if($permission){
            return redirect('admin/permission/index')->with('message', 'Permission updated successfully!');
        } else {
            return back()->withError('Something went wrong! Please try again.');
        }
    }

    /**
     * delete the Permission.
     *
     */
    public function destroy(Request $request)
    {
        $permission = Permission::findOrFail( $request->id );
        if($permission->delete()){
            return back()->with('message','Permission deleted successfully');
        } else {
            return back()->withError('Something went wrong! Please try again.');
        }
    }

    /**
     * assign permission.
     *
     */
    public function assign(Request $request)
    {
        $role = Role::find($request->roleid);
        return view('admin.role-permission.create', [
            'roles' => Role::all(),
            'permissions' => Permission::all(),
            'chcked_permissions' => $role ? $role->permissions()->pluck('id')->toArray() : [],
        ]);
    }

    /**
     * assign permission store in database.
     *
     */
    public function assignstore(Request $request)
    {
        $request->validate([
            'role' => ['required'],
        ]);
        $role = Role::find($request->role);
        $permission_assign = $request->permission ? $role->syncPermissions($request->permission) : $role->syncPermissions([]);
        if($permission_assign){
            return back()->with('success','Permission assigned successfully');
            // return redirect('admin/governorate')->with('success','Record has been created successfully!');
        } else {
            // return redirect('admin/user/create')->with('error','Record not saved !!');
            return back()->with('error','Something went wrong! Please try again.');
        }

    }
}
